@@ -11,4 +11,10 @@ Simple wrapper for the MyAnimeList API.
 __title__ = 'mal'
 __author__ = 'Skylake-dev'
 __license__ = 'MIT'
-__version__ = '0.4.1'
+__version__ = '0.5.1'
+
+import logging
+
+logging.getLogger(__name__).addHandler(logging.NullHandler())
+
+del logging
\ No newline at end of file
