@@ -1,14 +1,17 @@
 """Module in charge of making requests to the API and managing possible errors."""
 import requests
+import logging
 from typing import Any, List, Dict, Optional, Sequence, Union
 
 from .endpoints import Endpoint
 from .anime import Anime, AnimeSearchResults, AnimeList, AnimeRanking, Seasonal
 from .manga import Manga, MangaSearchResults, MangaList, MangaRanking
 from .forum import BoardCategory, ForumTopics, Discussion
-from .enums import Field, AnimeListStatus, MangaListStatus, Season, AnimeRankingType, MangaRankingType
+from .enums import (Field, AnimeListStatus, MangaListStatus, Season,
+                    AnimeRankingType, MangaRankingType, AnimeListSort, MangaListSort)
 from .utils import MISSING
 
+_log = logging.getLogger(__name__)
 
 class Client:
     """Offers the interface to make requests."""
@@ -33,6 +36,7 @@ class Client:
     @search_limit.setter
     def search_limit(self, value: int) -> None:
         self._search_limit = value if 0 < value < 100 else 100
+        _log.info(f'parameter "search_limit" default value set to {self._search_limit}')
 
     @property
     def anime_fields(self) -> List[Field]:
@@ -45,6 +49,7 @@ class Client:
     @anime_fields.setter
     def anime_fields(self, new_fields: Sequence[Union[Field, str]]) -> None:
         self._anime_fields = Field.from_list(new_fields)
+        _log.info(f'parameter "anime_fields" default value set to {self.anime_fields}')
 
     @property
     def manga_fields(self) -> List[Field]:
@@ -57,6 +62,7 @@ class Client:
     @manga_fields.setter
     def manga_fields(self, new_fields: Sequence[Union[Field, str]]) -> None:
         self._manga_fields = Field.from_list(new_fields)
+        _log.info(f'parameter "manga_fields" default value set to {self.manga_fields}')
 
     @property
     def include_nsfw(self) -> bool:
@@ -69,6 +75,7 @@ class Client:
     @include_nsfw.setter
     def include_nsfw(self, value: bool) -> None:
         self._include_nsfw = value
+        _log.info(f'parameter "nsfw" default value set to {value}')
 
     def anime_search(
         self,
@@ -184,6 +191,7 @@ class Client:
         offset: int = MISSING,
         fields: Sequence[Union[Field, str]] = MISSING,
         status: Union[AnimeListStatus, str] = MISSING,
+        sort: Union[AnimeListSort, str] = MISSING,
         include_nsfw: bool = MISSING
     ) -> AnimeList:
         """Returns the anime list of a specific user, if public.
@@ -196,13 +204,14 @@ class Client:
             offset: get results at a certain offset from the start, defaults to 0
             fields: set which fields to get for each entry
             status: return only a specific category. will return all if omitted
+            sort: specify the sorting of the list
             include_nsfw: include results marked as nsfw
 
         Returns:
             AnimeList: iterable with all the entries of the list
         """
         parameters = self._build_list_paramenters(
-            Endpoint.USER_ANIMELIST, limit=limit, offset=offset, fields=fields, status=status, nsfw=include_nsfw)
+            Endpoint.USER_ANIMELIST, limit=limit, offset=offset, fields=fields, status=status, nsfw=include_nsfw, sort=sort)
         url = Endpoint.USER_ANIMELIST.url.replace('{username}', username)
         response = self._request(url, params=parameters)
         data = response.json()
@@ -216,6 +225,7 @@ class Client:
         offset: int = MISSING,
         fields: Sequence[Union[Field, str]] = MISSING,
         status: Union[MangaListStatus, str] = MISSING,
+        sort: Union[MangaListSort, str] = MISSING,
         include_nsfw: bool = MISSING
     ) -> MangaList:
         """Returns the manga list of a specific user, if public.
@@ -228,13 +238,14 @@ class Client:
             offset: get results at a certain offset from the start, defaults to 0
             fields: set which fields to get for each entry
             status: return only a specific category. will return all if omitted
+            sort: specify the sorting of the list
             include_nsfw: include results marked as nsfw
 
         Returns:
             MangaList: iterable with all the entries of the list
         """
         parameters = self._build_list_paramenters(
-            Endpoint.USER_MANGALIST, limit=limit, offset=offset, fields=fields, status=status, nsfw=include_nsfw)
+            Endpoint.USER_MANGALIST, limit=limit, offset=offset, fields=fields, status=status, nsfw=include_nsfw, sort=sort)
         url = Endpoint.USER_MANGALIST.url.replace('{username}', username)
         response = self._request(url, params=parameters)
         data = response.json()
@@ -423,7 +434,10 @@ class Client:
         if url is None:
             return None
         response = self._session.get(url)
-        response.raise_for_status()
+        _log.info(f'Fetching url: {url}')
+        if response.status_code != requests.codes.ok:
+            _log.error(f'Request to {url} errored with code {response.status_code}')
+            response.raise_for_status()
         return response.json()
 
     def _request(self, url: str, params: Dict[str, str] = MISSING) -> requests.Response:
@@ -434,7 +448,9 @@ class Client:
             response = self._session.get(url, params=params)
         else:
             response = self._session.get(url)
-        response.raise_for_status()  # TODO: handle error and possible retries
+        if response.status_code != requests.codes.ok:
+            _log.error(f'Request to {url} with parameters {params} errored with code {response.status_code}')
+            response.raise_for_status()  # TODO: handle error and possible retries
         return response
 
     def _build_search_parameters(
@@ -484,6 +500,7 @@ class Client:
         offset: int = MISSING,
         fields: Sequence[Union[Field, str]] = MISSING,
         status: Union[AnimeListStatus, MangaListStatus, str] = MISSING,
+        sort: Union[AnimeListSort, MangaListSort, str] = MISSING,
         nsfw: bool = MISSING
     ) -> Dict[str, str]:
         parameters: Dict[str, str] = {}
@@ -508,6 +525,12 @@ class Client:
             else:
                 value = status.value
             parameters['status'] = value
+        if sort is not MISSING:
+            if isinstance(sort, str):
+                value = sort   # NOTE: default sort is by title
+            else:
+                value = sort.value
+            parameters['sort'] = value
         # nsfw overrides the value stored in self.include_nsfw
         if nsfw is MISSING:
             if self.include_nsfw:
